unit uSyncBackground;

interface

uses
  Dialogs,
  SysUtils,
  Classes,
  uMain,
  IdFTP,
  IdFTPCommon,
  IdSocks,
  SyncObjs;

  procedure StartSyncBackground();

implementation

uses
  uThreadUpdateProgress;

//***************************************************************************//
//      FTP
function FileExistsRemote(FTP: TIdFTP; FileName: string): boolean;
var
  Files: TStrings;
  i: integer;
begin
  Files := TStringList.Create;
  try
    FTP.List(Files,'*.*',false);
    Result := false;
    i := 0;
    while Not Result and (i < Files.Count) do
      if CompareText(Files[i],FileName) = 0
        then Result := true
      else inc(i);
  finally
    Files.Free;
  end;
end;

//******************************************************************************//
//      
function LockFileTimedOut(LockFileName: string; UnlockWait: Integer): boolean;
var S: TStrings;
    ts: string;
    LockFileTime: TDateTime;
begin
     try
        S := TStringList.Create;
        try
           S.LoadFromFile(LockFileName);
           if S.Count = 0 then Result := true else
           begin
              ts := S[0];
              LockFileTime :=
                 EncodeDate(
                    StrToInt(Copy(ts,1,4)),
                    StrToInt(Copy(ts,5,2)),
                    StrToInt(Copy(ts,7,2))
                 ) +
                 EncodeTime(
                    StrToInt(Copy(ts,9,2)),
                    StrToInt(Copy(ts,11,2)),
                    StrToInt(Copy(ts,13,2)),
                    0
                 );
              Result := (Now - LockFileTime)*1000 >= UnlockWait;
           end;
        finally
           S.Free;
        end;
     except
        Result := false;
     end;
end;

//******************************************************************************
//      FTP
function GetTransferType(Str: string): TIdFTPTransferType;
begin
  if Str = 'ASCII' then Result := ftASCII else
  if Str = 'Binary'then Result := ftBinary else
    raise Exception.CreateFmt('   ', [Str]);
end;

//******************************************************************************
//          
function GetListLocalFiles(Dir: String; Mask: String): TStrings;
var
  FS: TSearchRec;
  StrMasks: TStrings;
  i: Integer;
begin
  StrMasks := StrRefresh(Mask, ';');
  Result := TStringList.Create();
  for i:=0 to StrMasks.Count-1 do
  begin
    if FindFirst(Dir + '\' + StrMasks[i], faAnyFile and Not faDirectory, FS) = 0 then
    begin
      Result.Add(FS.Name);
      while FindNext(FS) = 0 do
      begin
        Result.Add(FS.Name);
      end;
    end;
    FindClose(FS);
  end;
end;

//*********************************************************************************
//      FTP
procedure LockFile(LocalOutbox: String; LockExt: String; FileName: String; FTP: TIdFTP);
var
  S: TStrings;
  LockFileName: String;
begin
  if LockExt <> '' then
  begin
    LockFileName := FileName + '.' + LockExt;
    S := TStringList.Create;
    try
       S.Add(FormatDateTime('yyyymmddhhmmss',Now));
       S.SaveToFile(LocalOutbox + '\' + LockFileName);
    finally
       S.Free;
    end;
    FTP.Put(LocalOutbox + '\' + LockFileName, LockFileName);
    DeleteFile(LocalOutbox + '\' + LockFileName);
  end;
end;

//*********************************************************************************
//      FTP
procedure UnlockFile(LockExt: String; FileName: String; FTP: TIdFTP);
var
  LockFileName: String;
begin
  if LockExt <> '' then
  begin
    LockFileName := FileName + '.' + LockExt;
    FTP.Delete(LockFileName);
  end;
end;

//******************************************************************************//
//     
procedure WaitUnlock(LockExt: String; FileName: String; FTP: TIdFTP; UnlockWait: Integer);
var
  LockFileName: String;
begin
  if LockExt <> '' then
  begin
    LockFileName := FileName + '.' + LockExt;
    // ,      
    if FileExistsRemote(FTP, LockFileName) then
    begin
      //   , ,     
      //    
      FTP.Get(LockFileName, Dir + '\lock.tmp', True);
      while not (LockFileTimedOut(Dir + '\lock.tmp', UnlockWait)) and (FileExistsRemote(FTP, LockFileName)) do
      begin
        Sleep(5000);
      end;
      DeleteFile(Dir + '\lock.tmp');
      if FileExistsRemote(FTP, LockFileName) then
      begin
        try
          FTP.Delete(LockFileName);
        except
          Raise Exception.CreateFmt('     ' + FileName, [LockFileName]);
        end;
      end;
    end;
  end;
end;

//******************************************************************************
//    
procedure Synchronize(SingeSync: SettingsSync; var Error: Boolean);
var
  Files: TStrings;
  FTP: TIdFTP;
  i, l : Integer;
  ActionStr: String;
  DirFTP: TStrings;
  StrMasks: TStrings;
  FilesCount: Integer;
begin
  try
    //  FTP 
    ActionStr := TimeToStr(Now) + '   FTP...' + #13#10;
    FTP := TIdFTP.Create(nil);
    FTP.Host := SingeSync.Host;
    FTP.Port := SingeSync.Port;
    FTP.Passive := SingeSync.Passive;
    FTP.Username := SingeSync.User;
    FTP.Password := SingeSync.Password;
    FTP.TransferType := GetTransferType(SingeSync.TransferType);
    FTP.Connect();

    ActionStr := ActionStr + TimeToStr(Now) + '  ...' + #13#10;
    //     FTP
    Files := TStringList.Create();

    //   
    DirFTP := StrRefresh(SingeSync.RemoteOutbox, '\');
    for i:=0 to DirFTP.Count-1 do
    begin
      FTP.ChangeDir(DirFTP[i]);
    end;

    StrMasks := StrRefresh(SingeSync.RemoteMask, ';');
    for l:=0 to StrMasks.Count-1 do
    begin
      FTP.List(Files, StrMasks[l], False);

      //     
      i := 0;
      while i <= Files.Count - 1 do
      begin
        if copy(Files[i], Length(Files[i])-2, 4) = SingeSync.LockExt then
        begin
          Files.Delete(i);
        end else Inc(i);
      end;

      if Files.Count <> 0 then
      begin
        //  FTP     
        for i:=0 to Files.Count-1 do
        begin
          //  ,    
          WaitUnlock(SingeSync.LockExt, Files[i], FTP, SingeSync.UnlockWait);
          //  
          FTP.Get(Files[i], SingeSync.LocalInbox + '\' + Files[i], True);
          //    FTP
          FTP.Delete(Files[i]);
          Inc(FilesCount);
        end;
      end;
      Files.Clear;
    end;

    if FilesCount > 0 then
      ActionStr := ActionStr + TimeToStr(Now) + '  : ' + IntToStr(FilesCount)
    else
      ActionStr := ActionStr + TimeToStr(Now) + '    .';

    FTP.Disconnect();
    FTP.Connect();

    ActionStr := ActionStr + #13#10 + TimeToStr(Now) + '  ...' + #13#10;
    //      
    Files := GetListLocalFiles(SingeSync.LocalOutbox, SingeSync.LocalMask);
    if Files.Count <> 0 then
    begin
      //      FTP
      //     FTP

      //   
      DirFTP := StrRefresh(SingeSync.RemoteInbox, '\');
      for i:=0 to DirFTP.Count-1 do
      begin
        FTP.ChangeDir(DirFTP[i]);
      end;

      for i:=0 to Files.Count-1 do
      begin
        //   
        LockFile(SingeSync.LocalOutbox, SingeSync.LockExt, Files[i], FTP);
        //  
        FTP.Put(SingeSync.LocalOutbox + '\' + Files[i], Files[i]);
        //   
        UnlockFile(SingeSync.LockExt, Files[i], FTP);
        //     
        DeleteFile(SingeSync.LocalOutbox + '\' + Files[i]);
      end;
      ActionStr := ActionStr + TimeToStr(Now) + '  : ' + IntToStr(i);
    end else
    begin
     ActionStr := ActionStr + TimeToStr(Now) + '    .';
    end;

    WriteLog(52, 'C  : ' + SingeSync.Name + '  .', ActionStr, False);
  except
  on E:Exception do
    begin
      WriteLog(52, '   : ' + SingeSync.Name, ActionStr + E.Message, True);
      Error := True;
      Exit;
    end;
  end;
end;

//******************************************************************************
//   FTP
procedure StartSyncBackground();
var
  countSync, i: Byte;
  Error: Boolean;
begin
  //     
  countSync := 0;
  for i:=0 to MainData.CountSync-1 do
  begin
    if MainData.DataSync[i].Exchange then
      Inc(countSync);
  end;

  if countSync = 0 then
  begin
    //   
    WriteLog(52, '  FTP', '      ', True);
    Exit;
  end;

  //       
  for i:=0 to MainData.CountSync-1 do
  begin
    if not MainData.DataSync[i].Exchange then
    begin
      Continue;
    end;
    Synchronize(MainData.DataSync[i], Error);
  end;
end;

end.
