unit uThreadExchange;

interface

uses
  Classes, SyncObjs, ComObj, SysUtils, Windows, uMain, Forms, ActiveX, IdFTP,
  IdFTPCommon;

type
  TThreadExchange = class(TThread)
  private
    FinallyExchange: Byte; //    
    NeedUpdate: Boolean; //   
    LastError: String; //     ,      
    StartPing: Boolean; //  ,    
  public
    function GetTerminated(): Boolean;
  protected
    function CreateComConnection(SingleSetting: Settings):Variant;
    procedure Run1CBase(SingleSetting: Settings);
    procedure UpdateCfg(SingleSetting: Settings);
    procedure StartSingleExchange(i: Byte; var Error: Boolean);
    procedure Execute; override;
  end;

var
  ThreadExchange: TThreadExchange;
  CriticalExchange: TCriticalSection;
  FTP: TIdFTP;
  ConnectionObj: Variant;

implementation

uses
  uThreadUpdateProgress, uThreadPing;

function TThreadExchange.GetTerminated(): Boolean;
begin
  Result:=Terminated;
end;

//*****************************************************************************//
//    COM
function TThreadExchange.CreateComConnection(SingleSetting: Settings):Variant;
var
  ConnectionObj: Variant;
  ConnectionString: String;
  SrvrStr: TStrings;
begin
  //   1
  CoInitialize(nil);
  ConnectionObj:=CreateOleObject('V81.Application');
  if SingleSetting.DBType = 0 then
  begin
    ConnectionString:='file='+ SingleSetting.DBPath+';'+' usr='+SingleSetting.DBUserName+';'+' pwd='+SingleSetting.DBUserPassword;
  end else
  begin
    //       
    SrvrStr := StrRefresh(SingleSetting.DBPath, '\');
    ConnectionString:='srvr='+ SrvrStr[0]+';'+ ' ref=' + SrvrStr[1] +  ';' + ' usr='+SingleSetting.DBUserName+';'+' pwd='+SingleSetting.DBUserPassword;
  end;
  Result:=ConnectionObj.Connect(ConnectionString);
  CoUninitialize;
end;

///////////////////////////////////////////////////////////////////////////////
//    
procedure TThreadExchange.UpdateCfg(SingleSetting: Settings);
var
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
  UpdateString: String;
begin
  //   
  FillChar(StartupInfo, SizeOf(StartupInfo), #0);
  StartupInfo.cb:=SizeOf(StartupInfo);
  //     
  UpdateString:= MainData.RunPath + ' CONFIG /F' + SingleSetting.DBPath + ' /N' +
               SingleSetting.DBUserName + ' /P' + SingleSetting.DBUserPassword + ' /UpdateDBCfg';

  //  
  CreateProcess(nil, PChar(UpdateString), nil, nil, false, CREATE_NEW_CONSOLE, nil, nil, StartupInfo, ProcessInfo);

  ShowTextMain('  ' + #13#10 + '"' + SingleSetting.Name + '"' + #13#10 + '  ');

  while WaitForSingleObject(ProcessInfo.hProcess, 500) <> WAIT_OBJECT_0 do
  begin
    Sleep(1000);
  end;
end;

///////////////////////////////////////////////////////////////////////////////
//       
procedure TThreadExchange.Run1CBase(SingleSetting: Settings);

var
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
  StartString: String;

begin
  //   
  FillChar(StartupInfo, SizeOf(StartupInfo), #0);
  StartupInfo.cb:=SizeOf(StartupInfo);
  //    
  StartString:= MainData.RunPath + ' ENTERPRISE /F' + SingleSetting.DBPath + ' /N' +
               SingleSetting.AfterUserName + ' /P' + SingleSetting.AfterUserPassword;

  //  
  CreateProcess(nil, PChar(StartString), nil, nil, false, CREATE_NEW_CONSOLE, nil, nil, StartupInfo, ProcessInfo);

  //  
  WaitForSingleObject(ProcessInfo.hProcess, 10000);
end;

//*****************************************************************************//
//     
procedure TThreadExchange.StartSingleExchange(i: Byte; var Error: Boolean);
var
  ErrString: String[255]; //     
  SingleSetting: Settings;
  Connection : Variant; // Com-
  SetExchange: Variant; //  -     ""
  countConn: Byte; //     
begin
  SingleSetting := MainData.Data[i];

  //     
  try
    ShowTextMain('  ' + #13#10 + '"' + SingleSetting.Name + '"' + #13#10 + '   ');
    Connection := CreateComConnection(SingleSetting);
  except
  on E:Exception do
    begin
      ErrString := ' !' + #13#10 + E.Message;
      WriteLog(i, '     ', ErrString, True);
      Error := True;
      LastError := ErrString;
      ShowTextMain(ErrString);
      Exit;
    end;
  end;

  ShowTextMain('   ' + #13#10 + '"' + SingleSetting.Name + '"');

  //     
  try
    SetExchange := Connection...(SingleSetting.ExchangeCode);
    if SetExchange.() then
    begin
      ErrString := ' !' + #13#10 + '    ' + SingleSetting.ExchangeCode + '  !';
      WriteLog(i, '   ', ErrString, True);
      ShowTextMain(ErrString);
      LastError := ErrString;
      Error := True;
      Exit;
    end;
  except
  on E:Exception do
    begin
      ErrString := ' !' + #13#10 + '     !';
      WriteLog(i, '   ', '     !' + #13#10 + E.Message, True);
      ShowTextMain(ErrString);
      LastError := ErrString;
      Error := True;
      Connection := '';
      Exit;
    end;
  end;

  //     
	countConn := Connection.().();
  if countConn > 1 then
  begin
    if not SingleSetting.RunExchangeIfRun then
    begin
      ErrString := '      ' + #13#10 +
                            '   !' + #13#10 +
                          '       .';

      WriteLog(i, '   ', '    ' + #13#10 + ' : ' + IntToStr(countConn-1), True);
      ShowTextMain(ErrString);
      LastError := ErrString;
      Error := True;
      Connection := '';
      Exit;
    end;
  end;

  //        
  Connection..(SetExchange);

  //  ,  -    
  NeedUpdate := Connection.();

  if NeedUpdate then
  begin
    countConn := Connection.().();
    if countConn > 1 then
    begin
      //    
      if SingleSetting.WaitExchange then
      begin
        //        
        countConn := Connection.().();
        while countConn > 1 do
        begin
          //     30   
          ShowTextMain('  ' + #13#10 + '"' + SingleSetting.Name + '"' + #13#10 + '       ');
          Sleep(30000);
          countConn := Connection.().();
        end;
      end else
      begin
        //  
        ErrString := ' !' + #13#10 + '    ' + #13#10 + ' : ' + IntToStr(countConn-1);
        WriteLog(i, '    ', ErrString, True);
        Error := True;
        ShowTextMain(ErrString);
        LastError := ErrString;
        Exit;
      end;
    end
  end;
end;

//*****************************************************************************//
//    
procedure TThreadExchange.Execute;
var
  i, l, DownLog: Word;
  ErrString: String[255];
  countExchange: Byte; //     
  FS: TFileStream; //       
  LogPath: String;
  LogSize: Int64;
  Error: Boolean;
  TempLog: TextFile;
begin
  //   
  CriticalExchange.Enter;
  TerminatedProgress:=False;
  try
    countExchange := 0;
    //     
    if not FileExists(MainData.RunPath) then
    begin
      ErrString := ' ' + #13#10 + '   1 ';
      WriteLog(51, '   ', ErrString, True);
      ShowTextMain(ErrString);
      Raise Exception.CreateFmt(ErrString, ['']);
    end;

    if MainData.Count = 0 then
    begin
      ErrString := '     !';
      ShowTextMain(ErrString);
      WriteLog(51, '', '  :' + #13#10 + ErrString, True);
      Raise Exception.CreateFmt(ErrString, ['']);
    end;

    // ,      
    for i:=0 to MainData.Count - 1 do
    begin
      if not MainData.Data[i].Exchange then
      begin
        //   
        Continue;
      end;
      Inc(countExchange);
    end;

    if countExchange = 0 then
    begin
      ErrString := '     !';
      ShowTextMain(ErrString);
      WriteLog(51, '', '  :' + #13#10 + ErrString, True);
      Raise Exception.CreateFmt(ErrString, ['']);
    end;

    //     
    if MainData.Ping then
    begin;
      ShowTextMain('  ');
      ErrString := CreatePing();
      if ErrString <> '' then
      begin
        ErrString := ' !' + #13#10 + '    ' + #13#10 + ErrString;
        WriteLog(51, '   ', ErrString, True);
        ShowTextMain(ErrString);
        Raise Exception.CreateFmt(ErrString, ['']);
      end;
      //   
      ThreadPing := TThreadPing.Create(True);
      ThreadPing.Resume;
      StartPing := True;
    end;

    //     
    for i:=0 to MainData.Count - 1 do
    begin
      if not MainData.Data[i].Exchange then
      begin
        //   
        Continue;
      end;

      if IconInTray then
      begin
        //   
        LastTrayTip := '   ' + #13#10 + '"' + MainData.Data[i].Name + '"';
        TraySet(3, LastTrayTip);
      end;

      ShowTextMain('    ' + #13#10 + '"' + MainData.Data[i].Name + '"');

      StartSingleExchange(i, Error);

      if NeedUpdate and not Error then
      begin
        //   
        try
          UpdateCfg(MainData.Data[i]);
          //   
          StartSingleExchange(i, Error);
        except
        on E:Exception do
          begin
            ErrString := ' !' + #13#10 + E.Message;
            WriteLog(i, '    ', ErrString, True);
            ShowTextMain(ErrString);
            LastError := ErrString;
            Error := True;
          end;
        end;
      end;
      if not Error then
      begin
        if MainData.Data[i].RunAfterExchange then
        begin
          //         1 
          ShowTextMain('    ');
          Run1CBase(MainData.Data[i]);
        end;
        //     
        WriteLog(i, MainData.Data[i].Name + '.    !', '    ' + MainData.Data[i].Name + '  !', False);
        Inc(FinallyExchange);
      end;
    end;

    //     FTP
    try
      LogPath := MainData.LogPath;
      if MainData.SendLogToFTP then
      begin
        FTP := TIdFTP.Create(nil);
        FTP.Host := MainData.FTPServer;
        FTP.Username := MainData.FTPUserName;
        FTP.Password := MainData.FTPUserPassword;
        FTP.Passive := True;
        if FileExists(LogPath) and not Error then
        begin
          ShowTextMain('   FTP ');
          FTP.Connect();

          if MainData.FTPDir <> '' then
          begin
            FTP.ChangeDir(MainData.FTPDir);
          end;

          //    
          FTP.Put(LogPath, 'Exchange.log');

          //   
          if MainData.DeleteLog then
          begin
            FS := TFileStream.Create(LogPath, fmOpenRead);
            LogSize := FS.Size;
            FS.Free;
            if LogSize/1024 > MainData.FileLogSize then
            begin
              DeleteFile(PChar(LogPath));
            end;
          end;
        end else
        begin
          if Error then
          begin
            ShowTextMain('    FTP ');
            //      FTP   15   
            FTP.Connect();

            if MainData.FTPDir <> '' then
            begin
              FTP.ChangeDir(MainData.FTPDir);
            end;

            //    
            AssignFile(TempLog, Dir+'\RBDExchange.log');
            Rewrite(TempLog);
            if LogsData.Count > 16 then
            begin
              DownLog := LogsData.Count - 16;
            end else
            begin
              DownLog := 0;
            end;

            for l:=DownLog to LogsData.Count-1 do
            begin
              WriteLn(TempLog, LogsData.Data[l].Date + ' ' + LogsData.Data[l].Time + ' ' + LogsData.Data[l].Text + ' ' + LogsData.Data[l].Description);
            end;
            CloseFile(TempLog);

            //   FTP
            FTP.Put(Dir+'\RBDExchange.log', 'RBDExchange.log');

            //    
            DeleteFile(PChar(Dir+'\RBDExchange.log'));
          end;
        end;
      end;
    except
    on E:Exception do
      begin
        ErrString := '      FTP' + #13#10 + E.Message;
        WriteLog(i, '      FTP', ErrString, True);
      end;
    end;

    if Error and (countExchange > 1) then
    begin
      ShowTextMain('   c ! . .');
    end else
    begin
      if not Error then
      begin
        if countExchange > 1 then
        begin
          ShowTextMain('  ' + IntToStr(FinallyExchange) + ' ' + IntToStr(countExchange) +
                         #13#10 + '. ');
        end else
        begin
          ShowTextMain('   !');
        end;
      end else
      begin
        ShowTextMain(LastError);
      end;
    end;
  finally
    if IconInTray then
    begin
      //     
      LastTrayTip := '';
      TraySet(3, '');
    end;
    TerminatedProgress:=True;
    //   
    CriticalExchange.Leave;
    //   ,  
    if StartPing then
    begin
      ThreadPing.Terminate;
    end;
    Terminate;
    // ,    
    if MainData.CloseAfterExchange then
    begin
      SaveLogs();
      Application.Terminate();
    end;
  end;
end;

end.
